/* Copyright (C) 2020 Google, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

#include "tu_tracepoints.h"

#include "vk_enum_to_str.h"
#include "tu_cmd_buffer.h"
#include "tu_device.h"

#define __NEEDS_TRACE_PRIV
#include "util/u_debug.h"
#include "util/perf/u_trace_priv.h"

static const struct debug_control config_control[] = {
   { "cmd_buffer", TU_GPU_TRACEPOINT_CMD_BUFFER, },
   { "render_pass", TU_GPU_TRACEPOINT_RENDER_PASS, },
   { "binning_ib", TU_GPU_TRACEPOINT_BINNING_IB, },
   { "draw_ib_sysmem", TU_GPU_TRACEPOINT_DRAW_IB_SYSMEM, },
   { "draw_ib_gmem", TU_GPU_TRACEPOINT_DRAW_IB_GMEM, },
   { "gmem_clear", TU_GPU_TRACEPOINT_GMEM_CLEAR, },
   { "sysmem_clear", TU_GPU_TRACEPOINT_SYSMEM_CLEAR, },
   { "sysmem_clear_all", TU_GPU_TRACEPOINT_SYSMEM_CLEAR_ALL, },
   { "gmem_load", TU_GPU_TRACEPOINT_GMEM_LOAD, },
   { "gmem_store", TU_GPU_TRACEPOINT_GMEM_STORE, },
   { "sysmem_resolve", TU_GPU_TRACEPOINT_SYSMEM_RESOLVE, },
   { "blit", TU_GPU_TRACEPOINT_BLIT, },
   { "compute", TU_GPU_TRACEPOINT_COMPUTE, },
   { NULL, 0, },
};
uint64_t tu_gpu_tracepoint = 0;

static void
tu_gpu_tracepoint_variable_once(void)
{
   uint64_t default_value = 0
     | TU_GPU_TRACEPOINT_CMD_BUFFER
     | TU_GPU_TRACEPOINT_RENDER_PASS
     | TU_GPU_TRACEPOINT_BINNING_IB
     | TU_GPU_TRACEPOINT_DRAW_IB_SYSMEM
     | TU_GPU_TRACEPOINT_DRAW_IB_GMEM
     | TU_GPU_TRACEPOINT_GMEM_CLEAR
     | TU_GPU_TRACEPOINT_SYSMEM_CLEAR
     | TU_GPU_TRACEPOINT_SYSMEM_CLEAR_ALL
     | TU_GPU_TRACEPOINT_GMEM_LOAD
     | TU_GPU_TRACEPOINT_GMEM_STORE
     | TU_GPU_TRACEPOINT_SYSMEM_RESOLVE
     | TU_GPU_TRACEPOINT_BLIT
     | TU_GPU_TRACEPOINT_COMPUTE
     ;

   tu_gpu_tracepoint =
      parse_enable_string(getenv("TU_GPU_TRACEPOINT"),
                          default_value,
                          config_control);
}

void
tu_gpu_tracepoint_config_variable(void)
{
   static once_flag process_tu_gpu_tracepoint_variable_flag = ONCE_FLAG_INIT;

   call_once(&process_tu_gpu_tracepoint_variable_flag,
             tu_gpu_tracepoint_variable_once);
}

/*
 * start_cmd_buffer
 */
#define __print_start_cmd_buffer NULL
#define __print_json_start_cmd_buffer NULL
static const struct u_tracepoint __tp_start_cmd_buffer = {
    ALIGN_POT(sizeof(struct trace_start_cmd_buffer), 8),   /* keep size 64b aligned */
    "start_cmd_buffer",
    false,
    __print_start_cmd_buffer,
    __print_json_start_cmd_buffer,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_start_cmd_buffer,
#endif
};
void __trace_start_cmd_buffer(
     struct u_trace *ut
   , void *cs
) {
   struct trace_start_cmd_buffer *__entry =
      (struct trace_start_cmd_buffer *)u_trace_append(ut, cs, &__tp_start_cmd_buffer);
   (void)__entry;
}

/*
 * end_cmd_buffer
 */
static void __print_end_cmd_buffer(FILE *out, const void *arg) {
   const struct trace_end_cmd_buffer *__entry =
      (const struct trace_end_cmd_buffer *)arg;
   fprintf(out, ""
      "level=%s, "
      "render_pass_continue=%u, "
         "\n"
   ,vk_CommandBufferLevel_to_str(__entry->level)
   ,__entry->render_pass_continue
   );
}

static void __print_json_end_cmd_buffer(FILE *out, const void *arg) {
   const struct trace_end_cmd_buffer *__entry =
      (const struct trace_end_cmd_buffer *)arg;
   fprintf(out, ""
      "\"level\": \"%s\""
         ", "
      "\"render_pass_continue\": \"%u\""
   ,vk_CommandBufferLevel_to_str(__entry->level)
   ,__entry->render_pass_continue
   );
}

static const struct u_tracepoint __tp_end_cmd_buffer = {
    ALIGN_POT(sizeof(struct trace_end_cmd_buffer), 8),   /* keep size 64b aligned */
    "end_cmd_buffer",
    false,
    __print_end_cmd_buffer,
    __print_json_end_cmd_buffer,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_end_cmd_buffer,
#endif
};
void __trace_end_cmd_buffer(
     struct u_trace *ut
   , void *cs
   , const struct tu_cmd_buffer * cmd
) {
   struct trace_end_cmd_buffer *__entry =
      (struct trace_end_cmd_buffer *)u_trace_append(ut, cs, &__tp_end_cmd_buffer);
   __entry->level = cmd->vk.level;
   __entry->render_pass_continue = !!(cmd->usage_flags & VK_COMMAND_BUFFER_USAGE_RENDER_PASS_CONTINUE_BIT);
}

/*
 * start_render_pass
 */
#define __print_start_render_pass NULL
#define __print_json_start_render_pass NULL
static const struct u_tracepoint __tp_start_render_pass = {
    ALIGN_POT(sizeof(struct trace_start_render_pass), 8),   /* keep size 64b aligned */
    "start_render_pass",
    false,
    __print_start_render_pass,
    __print_json_start_render_pass,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_start_render_pass,
#endif
};
void __trace_start_render_pass(
     struct u_trace *ut
   , void *cs
) {
   struct trace_start_render_pass *__entry =
      (struct trace_start_render_pass *)u_trace_append(ut, cs, &__tp_start_render_pass);
   (void)__entry;
}

/*
 * end_render_pass
 */
static void __print_end_render_pass(FILE *out, const void *arg) {
   const struct trace_end_render_pass *__entry =
      (const struct trace_end_render_pass *)arg;
   fprintf(out, ""
      "width=%u, "
      "height=%u, "
      "MRTs=%u, "
      "numberOfBins=%u, "
      "binWidth=%u, "
      "binHeight=%u, "
         "\n"
   ,__entry->width
   ,__entry->height
   ,__entry->MRTs
   ,__entry->numberOfBins
   ,__entry->binWidth
   ,__entry->binHeight
   );
}

static void __print_json_end_render_pass(FILE *out, const void *arg) {
   const struct trace_end_render_pass *__entry =
      (const struct trace_end_render_pass *)arg;
   fprintf(out, ""
      "\"width\": \"%u\""
         ", "
      "\"height\": \"%u\""
         ", "
      "\"MRTs\": \"%u\""
         ", "
      "\"numberOfBins\": \"%u\""
         ", "
      "\"binWidth\": \"%u\""
         ", "
      "\"binHeight\": \"%u\""
   ,__entry->width
   ,__entry->height
   ,__entry->MRTs
   ,__entry->numberOfBins
   ,__entry->binWidth
   ,__entry->binHeight
   );
}

static const struct u_tracepoint __tp_end_render_pass = {
    ALIGN_POT(sizeof(struct trace_end_render_pass), 8),   /* keep size 64b aligned */
    "end_render_pass",
    false,
    __print_end_render_pass,
    __print_json_end_render_pass,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_end_render_pass,
#endif
};
void __trace_end_render_pass(
     struct u_trace *ut
   , void *cs
   , const struct tu_framebuffer * fb
   , const struct tu_tiling_config * tiling
) {
   struct trace_end_render_pass *__entry =
      (struct trace_end_render_pass *)u_trace_append(ut, cs, &__tp_end_render_pass);
   __entry->width = fb->width;
   __entry->height = fb->height;
   __entry->MRTs = fb->attachment_count;
   __entry->numberOfBins = tiling->tile_count.width * tiling->tile_count.height;
   __entry->binWidth = tiling->tile0.width;
   __entry->binHeight = tiling->tile0.height;
}

/*
 * start_binning_ib
 */
#define __print_start_binning_ib NULL
#define __print_json_start_binning_ib NULL
static const struct u_tracepoint __tp_start_binning_ib = {
    ALIGN_POT(sizeof(struct trace_start_binning_ib), 8),   /* keep size 64b aligned */
    "start_binning_ib",
    false,
    __print_start_binning_ib,
    __print_json_start_binning_ib,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_start_binning_ib,
#endif
};
void __trace_start_binning_ib(
     struct u_trace *ut
   , void *cs
) {
   struct trace_start_binning_ib *__entry =
      (struct trace_start_binning_ib *)u_trace_append(ut, cs, &__tp_start_binning_ib);
   (void)__entry;
}

/*
 * end_binning_ib
 */
#define __print_end_binning_ib NULL
#define __print_json_end_binning_ib NULL
static const struct u_tracepoint __tp_end_binning_ib = {
    ALIGN_POT(sizeof(struct trace_end_binning_ib), 8),   /* keep size 64b aligned */
    "end_binning_ib",
    false,
    __print_end_binning_ib,
    __print_json_end_binning_ib,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_end_binning_ib,
#endif
};
void __trace_end_binning_ib(
     struct u_trace *ut
   , void *cs
) {
   struct trace_end_binning_ib *__entry =
      (struct trace_end_binning_ib *)u_trace_append(ut, cs, &__tp_end_binning_ib);
   (void)__entry;
}

/*
 * start_draw_ib_sysmem
 */
#define __print_start_draw_ib_sysmem NULL
#define __print_json_start_draw_ib_sysmem NULL
static const struct u_tracepoint __tp_start_draw_ib_sysmem = {
    ALIGN_POT(sizeof(struct trace_start_draw_ib_sysmem), 8),   /* keep size 64b aligned */
    "start_draw_ib_sysmem",
    false,
    __print_start_draw_ib_sysmem,
    __print_json_start_draw_ib_sysmem,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_start_draw_ib_sysmem,
#endif
};
void __trace_start_draw_ib_sysmem(
     struct u_trace *ut
   , void *cs
) {
   struct trace_start_draw_ib_sysmem *__entry =
      (struct trace_start_draw_ib_sysmem *)u_trace_append(ut, cs, &__tp_start_draw_ib_sysmem);
   (void)__entry;
}

/*
 * end_draw_ib_sysmem
 */
#define __print_end_draw_ib_sysmem NULL
#define __print_json_end_draw_ib_sysmem NULL
static const struct u_tracepoint __tp_end_draw_ib_sysmem = {
    ALIGN_POT(sizeof(struct trace_end_draw_ib_sysmem), 8),   /* keep size 64b aligned */
    "end_draw_ib_sysmem",
    false,
    __print_end_draw_ib_sysmem,
    __print_json_end_draw_ib_sysmem,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_end_draw_ib_sysmem,
#endif
};
void __trace_end_draw_ib_sysmem(
     struct u_trace *ut
   , void *cs
) {
   struct trace_end_draw_ib_sysmem *__entry =
      (struct trace_end_draw_ib_sysmem *)u_trace_append(ut, cs, &__tp_end_draw_ib_sysmem);
   (void)__entry;
}

/*
 * start_draw_ib_gmem
 */
#define __print_start_draw_ib_gmem NULL
#define __print_json_start_draw_ib_gmem NULL
static const struct u_tracepoint __tp_start_draw_ib_gmem = {
    ALIGN_POT(sizeof(struct trace_start_draw_ib_gmem), 8),   /* keep size 64b aligned */
    "start_draw_ib_gmem",
    false,
    __print_start_draw_ib_gmem,
    __print_json_start_draw_ib_gmem,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_start_draw_ib_gmem,
#endif
};
void __trace_start_draw_ib_gmem(
     struct u_trace *ut
   , void *cs
) {
   struct trace_start_draw_ib_gmem *__entry =
      (struct trace_start_draw_ib_gmem *)u_trace_append(ut, cs, &__tp_start_draw_ib_gmem);
   (void)__entry;
}

/*
 * end_draw_ib_gmem
 */
#define __print_end_draw_ib_gmem NULL
#define __print_json_end_draw_ib_gmem NULL
static const struct u_tracepoint __tp_end_draw_ib_gmem = {
    ALIGN_POT(sizeof(struct trace_end_draw_ib_gmem), 8),   /* keep size 64b aligned */
    "end_draw_ib_gmem",
    false,
    __print_end_draw_ib_gmem,
    __print_json_end_draw_ib_gmem,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_end_draw_ib_gmem,
#endif
};
void __trace_end_draw_ib_gmem(
     struct u_trace *ut
   , void *cs
) {
   struct trace_end_draw_ib_gmem *__entry =
      (struct trace_end_draw_ib_gmem *)u_trace_append(ut, cs, &__tp_end_draw_ib_gmem);
   (void)__entry;
}

/*
 * start_gmem_clear
 */
#define __print_start_gmem_clear NULL
#define __print_json_start_gmem_clear NULL
static const struct u_tracepoint __tp_start_gmem_clear = {
    ALIGN_POT(sizeof(struct trace_start_gmem_clear), 8),   /* keep size 64b aligned */
    "start_gmem_clear",
    false,
    __print_start_gmem_clear,
    __print_json_start_gmem_clear,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_start_gmem_clear,
#endif
};
void __trace_start_gmem_clear(
     struct u_trace *ut
   , void *cs
) {
   struct trace_start_gmem_clear *__entry =
      (struct trace_start_gmem_clear *)u_trace_append(ut, cs, &__tp_start_gmem_clear);
   (void)__entry;
}

/*
 * end_gmem_clear
 */
static void __print_end_gmem_clear(FILE *out, const void *arg) {
   const struct trace_end_gmem_clear *__entry =
      (const struct trace_end_gmem_clear *)arg;
   fprintf(out, ""
      "format=%s, "
      "samples=%u, "
         "\n"
   ,vk_format_description(__entry->format)->short_name
   ,__entry->samples
   );
}

static void __print_json_end_gmem_clear(FILE *out, const void *arg) {
   const struct trace_end_gmem_clear *__entry =
      (const struct trace_end_gmem_clear *)arg;
   fprintf(out, ""
      "\"format\": \"%s\""
         ", "
      "\"samples\": \"%u\""
   ,vk_format_description(__entry->format)->short_name
   ,__entry->samples
   );
}

static const struct u_tracepoint __tp_end_gmem_clear = {
    ALIGN_POT(sizeof(struct trace_end_gmem_clear), 8),   /* keep size 64b aligned */
    "end_gmem_clear",
    false,
    __print_end_gmem_clear,
    __print_json_end_gmem_clear,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_end_gmem_clear,
#endif
};
void __trace_end_gmem_clear(
     struct u_trace *ut
   , void *cs
   , enum VkFormat format
   , uint8_t samples
) {
   struct trace_end_gmem_clear *__entry =
      (struct trace_end_gmem_clear *)u_trace_append(ut, cs, &__tp_end_gmem_clear);
   __entry->format = format;
   __entry->samples = samples;
}

/*
 * start_sysmem_clear
 */
#define __print_start_sysmem_clear NULL
#define __print_json_start_sysmem_clear NULL
static const struct u_tracepoint __tp_start_sysmem_clear = {
    ALIGN_POT(sizeof(struct trace_start_sysmem_clear), 8),   /* keep size 64b aligned */
    "start_sysmem_clear",
    false,
    __print_start_sysmem_clear,
    __print_json_start_sysmem_clear,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_start_sysmem_clear,
#endif
};
void __trace_start_sysmem_clear(
     struct u_trace *ut
   , void *cs
) {
   struct trace_start_sysmem_clear *__entry =
      (struct trace_start_sysmem_clear *)u_trace_append(ut, cs, &__tp_start_sysmem_clear);
   (void)__entry;
}

/*
 * end_sysmem_clear
 */
static void __print_end_sysmem_clear(FILE *out, const void *arg) {
   const struct trace_end_sysmem_clear *__entry =
      (const struct trace_end_sysmem_clear *)arg;
   fprintf(out, ""
      "format=%s, "
      "uses_3d_ops=%u, "
      "samples=%u, "
         "\n"
   ,vk_format_description(__entry->format)->short_name
   ,__entry->uses_3d_ops
   ,__entry->samples
   );
}

static void __print_json_end_sysmem_clear(FILE *out, const void *arg) {
   const struct trace_end_sysmem_clear *__entry =
      (const struct trace_end_sysmem_clear *)arg;
   fprintf(out, ""
      "\"format\": \"%s\""
         ", "
      "\"uses_3d_ops\": \"%u\""
         ", "
      "\"samples\": \"%u\""
   ,vk_format_description(__entry->format)->short_name
   ,__entry->uses_3d_ops
   ,__entry->samples
   );
}

static const struct u_tracepoint __tp_end_sysmem_clear = {
    ALIGN_POT(sizeof(struct trace_end_sysmem_clear), 8),   /* keep size 64b aligned */
    "end_sysmem_clear",
    false,
    __print_end_sysmem_clear,
    __print_json_end_sysmem_clear,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_end_sysmem_clear,
#endif
};
void __trace_end_sysmem_clear(
     struct u_trace *ut
   , void *cs
   , enum VkFormat format
   , uint8_t uses_3d_ops
   , uint8_t samples
) {
   struct trace_end_sysmem_clear *__entry =
      (struct trace_end_sysmem_clear *)u_trace_append(ut, cs, &__tp_end_sysmem_clear);
   __entry->format = format;
   __entry->uses_3d_ops = uses_3d_ops;
   __entry->samples = samples;
}

/*
 * start_sysmem_clear_all
 */
#define __print_start_sysmem_clear_all NULL
#define __print_json_start_sysmem_clear_all NULL
static const struct u_tracepoint __tp_start_sysmem_clear_all = {
    ALIGN_POT(sizeof(struct trace_start_sysmem_clear_all), 8),   /* keep size 64b aligned */
    "start_sysmem_clear_all",
    false,
    __print_start_sysmem_clear_all,
    __print_json_start_sysmem_clear_all,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_start_sysmem_clear_all,
#endif
};
void __trace_start_sysmem_clear_all(
     struct u_trace *ut
   , void *cs
) {
   struct trace_start_sysmem_clear_all *__entry =
      (struct trace_start_sysmem_clear_all *)u_trace_append(ut, cs, &__tp_start_sysmem_clear_all);
   (void)__entry;
}

/*
 * end_sysmem_clear_all
 */
static void __print_end_sysmem_clear_all(FILE *out, const void *arg) {
   const struct trace_end_sysmem_clear_all *__entry =
      (const struct trace_end_sysmem_clear_all *)arg;
   fprintf(out, ""
      "mrt_count=%u, "
      "rect_count=%u, "
         "\n"
   ,__entry->mrt_count
   ,__entry->rect_count
   );
}

static void __print_json_end_sysmem_clear_all(FILE *out, const void *arg) {
   const struct trace_end_sysmem_clear_all *__entry =
      (const struct trace_end_sysmem_clear_all *)arg;
   fprintf(out, ""
      "\"mrt_count\": \"%u\""
         ", "
      "\"rect_count\": \"%u\""
   ,__entry->mrt_count
   ,__entry->rect_count
   );
}

static const struct u_tracepoint __tp_end_sysmem_clear_all = {
    ALIGN_POT(sizeof(struct trace_end_sysmem_clear_all), 8),   /* keep size 64b aligned */
    "end_sysmem_clear_all",
    false,
    __print_end_sysmem_clear_all,
    __print_json_end_sysmem_clear_all,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_end_sysmem_clear_all,
#endif
};
void __trace_end_sysmem_clear_all(
     struct u_trace *ut
   , void *cs
   , uint8_t mrt_count
   , uint8_t rect_count
) {
   struct trace_end_sysmem_clear_all *__entry =
      (struct trace_end_sysmem_clear_all *)u_trace_append(ut, cs, &__tp_end_sysmem_clear_all);
   __entry->mrt_count = mrt_count;
   __entry->rect_count = rect_count;
}

/*
 * start_gmem_load
 */
#define __print_start_gmem_load NULL
#define __print_json_start_gmem_load NULL
static const struct u_tracepoint __tp_start_gmem_load = {
    ALIGN_POT(sizeof(struct trace_start_gmem_load), 8),   /* keep size 64b aligned */
    "start_gmem_load",
    false,
    __print_start_gmem_load,
    __print_json_start_gmem_load,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_start_gmem_load,
#endif
};
void __trace_start_gmem_load(
     struct u_trace *ut
   , void *cs
) {
   struct trace_start_gmem_load *__entry =
      (struct trace_start_gmem_load *)u_trace_append(ut, cs, &__tp_start_gmem_load);
   (void)__entry;
}

/*
 * end_gmem_load
 */
static void __print_end_gmem_load(FILE *out, const void *arg) {
   const struct trace_end_gmem_load *__entry =
      (const struct trace_end_gmem_load *)arg;
   fprintf(out, ""
      "format=%s, "
      "force_load=%u, "
         "\n"
   ,vk_format_description(__entry->format)->short_name
   ,__entry->force_load
   );
}

static void __print_json_end_gmem_load(FILE *out, const void *arg) {
   const struct trace_end_gmem_load *__entry =
      (const struct trace_end_gmem_load *)arg;
   fprintf(out, ""
      "\"format\": \"%s\""
         ", "
      "\"force_load\": \"%u\""
   ,vk_format_description(__entry->format)->short_name
   ,__entry->force_load
   );
}

static const struct u_tracepoint __tp_end_gmem_load = {
    ALIGN_POT(sizeof(struct trace_end_gmem_load), 8),   /* keep size 64b aligned */
    "end_gmem_load",
    false,
    __print_end_gmem_load,
    __print_json_end_gmem_load,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_end_gmem_load,
#endif
};
void __trace_end_gmem_load(
     struct u_trace *ut
   , void *cs
   , enum VkFormat format
   , uint8_t force_load
) {
   struct trace_end_gmem_load *__entry =
      (struct trace_end_gmem_load *)u_trace_append(ut, cs, &__tp_end_gmem_load);
   __entry->format = format;
   __entry->force_load = force_load;
}

/*
 * start_gmem_store
 */
#define __print_start_gmem_store NULL
#define __print_json_start_gmem_store NULL
static const struct u_tracepoint __tp_start_gmem_store = {
    ALIGN_POT(sizeof(struct trace_start_gmem_store), 8),   /* keep size 64b aligned */
    "start_gmem_store",
    false,
    __print_start_gmem_store,
    __print_json_start_gmem_store,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_start_gmem_store,
#endif
};
void __trace_start_gmem_store(
     struct u_trace *ut
   , void *cs
) {
   struct trace_start_gmem_store *__entry =
      (struct trace_start_gmem_store *)u_trace_append(ut, cs, &__tp_start_gmem_store);
   (void)__entry;
}

/*
 * end_gmem_store
 */
static void __print_end_gmem_store(FILE *out, const void *arg) {
   const struct trace_end_gmem_store *__entry =
      (const struct trace_end_gmem_store *)arg;
   fprintf(out, ""
      "format=%s, "
      "fast_path=%u, "
      "unaligned=%u, "
         "\n"
   ,vk_format_description(__entry->format)->short_name
   ,__entry->fast_path
   ,__entry->unaligned
   );
}

static void __print_json_end_gmem_store(FILE *out, const void *arg) {
   const struct trace_end_gmem_store *__entry =
      (const struct trace_end_gmem_store *)arg;
   fprintf(out, ""
      "\"format\": \"%s\""
         ", "
      "\"fast_path\": \"%u\""
         ", "
      "\"unaligned\": \"%u\""
   ,vk_format_description(__entry->format)->short_name
   ,__entry->fast_path
   ,__entry->unaligned
   );
}

static const struct u_tracepoint __tp_end_gmem_store = {
    ALIGN_POT(sizeof(struct trace_end_gmem_store), 8),   /* keep size 64b aligned */
    "end_gmem_store",
    false,
    __print_end_gmem_store,
    __print_json_end_gmem_store,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_end_gmem_store,
#endif
};
void __trace_end_gmem_store(
     struct u_trace *ut
   , void *cs
   , enum VkFormat format
   , uint8_t fast_path
   , uint8_t unaligned
) {
   struct trace_end_gmem_store *__entry =
      (struct trace_end_gmem_store *)u_trace_append(ut, cs, &__tp_end_gmem_store);
   __entry->format = format;
   __entry->fast_path = fast_path;
   __entry->unaligned = unaligned;
}

/*
 * start_sysmem_resolve
 */
#define __print_start_sysmem_resolve NULL
#define __print_json_start_sysmem_resolve NULL
static const struct u_tracepoint __tp_start_sysmem_resolve = {
    ALIGN_POT(sizeof(struct trace_start_sysmem_resolve), 8),   /* keep size 64b aligned */
    "start_sysmem_resolve",
    false,
    __print_start_sysmem_resolve,
    __print_json_start_sysmem_resolve,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_start_sysmem_resolve,
#endif
};
void __trace_start_sysmem_resolve(
     struct u_trace *ut
   , void *cs
) {
   struct trace_start_sysmem_resolve *__entry =
      (struct trace_start_sysmem_resolve *)u_trace_append(ut, cs, &__tp_start_sysmem_resolve);
   (void)__entry;
}

/*
 * end_sysmem_resolve
 */
static void __print_end_sysmem_resolve(FILE *out, const void *arg) {
   const struct trace_end_sysmem_resolve *__entry =
      (const struct trace_end_sysmem_resolve *)arg;
   fprintf(out, ""
      "format=%s, "
         "\n"
   ,vk_format_description(__entry->format)->short_name
   );
}

static void __print_json_end_sysmem_resolve(FILE *out, const void *arg) {
   const struct trace_end_sysmem_resolve *__entry =
      (const struct trace_end_sysmem_resolve *)arg;
   fprintf(out, ""
      "\"format\": \"%s\""
   ,vk_format_description(__entry->format)->short_name
   );
}

static const struct u_tracepoint __tp_end_sysmem_resolve = {
    ALIGN_POT(sizeof(struct trace_end_sysmem_resolve), 8),   /* keep size 64b aligned */
    "end_sysmem_resolve",
    false,
    __print_end_sysmem_resolve,
    __print_json_end_sysmem_resolve,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_end_sysmem_resolve,
#endif
};
void __trace_end_sysmem_resolve(
     struct u_trace *ut
   , void *cs
   , enum VkFormat format
) {
   struct trace_end_sysmem_resolve *__entry =
      (struct trace_end_sysmem_resolve *)u_trace_append(ut, cs, &__tp_end_sysmem_resolve);
   __entry->format = format;
}

/*
 * start_blit
 */
#define __print_start_blit NULL
#define __print_json_start_blit NULL
static const struct u_tracepoint __tp_start_blit = {
    ALIGN_POT(sizeof(struct trace_start_blit), 8),   /* keep size 64b aligned */
    "start_blit",
    false,
    __print_start_blit,
    __print_json_start_blit,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_start_blit,
#endif
};
void __trace_start_blit(
     struct u_trace *ut
   , void *cs
) {
   struct trace_start_blit *__entry =
      (struct trace_start_blit *)u_trace_append(ut, cs, &__tp_start_blit);
   (void)__entry;
}

/*
 * end_blit
 */
static void __print_end_blit(FILE *out, const void *arg) {
   const struct trace_end_blit *__entry =
      (const struct trace_end_blit *)arg;
   fprintf(out, ""
      "uses_3d_blit=%u, "
      "src_format=%s, "
      "dst_format=%s, "
      "layers=%u, "
         "\n"
   ,__entry->uses_3d_blit
   ,vk_format_description(__entry->src_format)->short_name
   ,vk_format_description(__entry->dst_format)->short_name
   ,__entry->layers
   );
}

static void __print_json_end_blit(FILE *out, const void *arg) {
   const struct trace_end_blit *__entry =
      (const struct trace_end_blit *)arg;
   fprintf(out, ""
      "\"uses_3d_blit\": \"%u\""
         ", "
      "\"src_format\": \"%s\""
         ", "
      "\"dst_format\": \"%s\""
         ", "
      "\"layers\": \"%u\""
   ,__entry->uses_3d_blit
   ,vk_format_description(__entry->src_format)->short_name
   ,vk_format_description(__entry->dst_format)->short_name
   ,__entry->layers
   );
}

static const struct u_tracepoint __tp_end_blit = {
    ALIGN_POT(sizeof(struct trace_end_blit), 8),   /* keep size 64b aligned */
    "end_blit",
    false,
    __print_end_blit,
    __print_json_end_blit,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_end_blit,
#endif
};
void __trace_end_blit(
     struct u_trace *ut
   , void *cs
   , uint8_t uses_3d_blit
   , enum VkFormat src_format
   , enum VkFormat dst_format
   , uint8_t layers
) {
   struct trace_end_blit *__entry =
      (struct trace_end_blit *)u_trace_append(ut, cs, &__tp_end_blit);
   __entry->uses_3d_blit = uses_3d_blit;
   __entry->src_format = src_format;
   __entry->dst_format = dst_format;
   __entry->layers = layers;
}

/*
 * start_compute
 */
#define __print_start_compute NULL
#define __print_json_start_compute NULL
static const struct u_tracepoint __tp_start_compute = {
    ALIGN_POT(sizeof(struct trace_start_compute), 8),   /* keep size 64b aligned */
    "start_compute",
    false,
    __print_start_compute,
    __print_json_start_compute,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_start_compute,
#endif
};
void __trace_start_compute(
     struct u_trace *ut
   , void *cs
) {
   struct trace_start_compute *__entry =
      (struct trace_start_compute *)u_trace_append(ut, cs, &__tp_start_compute);
   (void)__entry;
}

/*
 * end_compute
 */
static void __print_end_compute(FILE *out, const void *arg) {
   const struct trace_end_compute *__entry =
      (const struct trace_end_compute *)arg;
   fprintf(out, ""
      "indirect=%u, "
      "local_size_x=%u, "
      "local_size_y=%u, "
      "local_size_z=%u, "
      "num_groups_x=%u, "
      "num_groups_y=%u, "
      "num_groups_z=%u, "
         "\n"
   ,__entry->indirect
   ,__entry->local_size_x
   ,__entry->local_size_y
   ,__entry->local_size_z
   ,__entry->num_groups_x
   ,__entry->num_groups_y
   ,__entry->num_groups_z
   );
}

static void __print_json_end_compute(FILE *out, const void *arg) {
   const struct trace_end_compute *__entry =
      (const struct trace_end_compute *)arg;
   fprintf(out, ""
      "\"indirect\": \"%u\""
         ", "
      "\"local_size_x\": \"%u\""
         ", "
      "\"local_size_y\": \"%u\""
         ", "
      "\"local_size_z\": \"%u\""
         ", "
      "\"num_groups_x\": \"%u\""
         ", "
      "\"num_groups_y\": \"%u\""
         ", "
      "\"num_groups_z\": \"%u\""
   ,__entry->indirect
   ,__entry->local_size_x
   ,__entry->local_size_y
   ,__entry->local_size_z
   ,__entry->num_groups_x
   ,__entry->num_groups_y
   ,__entry->num_groups_z
   );
}

static const struct u_tracepoint __tp_end_compute = {
    ALIGN_POT(sizeof(struct trace_end_compute), 8),   /* keep size 64b aligned */
    "end_compute",
    false,
    __print_end_compute,
    __print_json_end_compute,
#ifdef HAVE_PERFETTO
    (void (*)(void *pctx, uint64_t, const void *, const void *))tu_end_compute,
#endif
};
void __trace_end_compute(
     struct u_trace *ut
   , void *cs
   , uint8_t indirect
   , uint16_t local_size_x
   , uint16_t local_size_y
   , uint16_t local_size_z
   , uint16_t num_groups_x
   , uint16_t num_groups_y
   , uint16_t num_groups_z
) {
   struct trace_end_compute *__entry =
      (struct trace_end_compute *)u_trace_append(ut, cs, &__tp_end_compute);
   __entry->indirect = indirect;
   __entry->local_size_x = local_size_x;
   __entry->local_size_y = local_size_y;
   __entry->local_size_z = local_size_z;
   __entry->num_groups_x = num_groups_x;
   __entry->num_groups_y = num_groups_y;
   __entry->num_groups_z = num_groups_z;
}

